@@ -26,7 +26,8 @@ branches and make a code block out of it. But it doesn't contain any target
 language syntax.
 """
 
-from nuitka.build.DataComposerInterface import deriveModuleConstantsBlobName
+from nuitka.plugins.Plugins import Plugins
+from nuitka.utils.CStrings import encodePythonStringToC
 
 from . import Contexts
 from .AsyncgenCodes import (
@@ -79,17 +80,14 @@ from .ComparisonCodes import (
     generateBuiltinIsinstanceCode,
     generateBuiltinIssubclassCode,
     generateComparisonExpressionCode,
+    generateTypeCheckCode,
 )
 from .ConditionalCodes import (
     generateConditionalAndOrCode,
     generateConditionalCode,
 )
 from .ConstantCodes import (
-    generateConstantEllipsisReferenceCode,
-    generateConstantFalseReferenceCode,
-    generateConstantNoneReferenceCode,
     generateConstantReferenceCode,
-    generateConstantTrueReferenceCode,
     getConstantsDefinitionCode,
 )
 from .CoroutineCodes import (
@@ -468,7 +466,9 @@ def generateModuleCode(module, data_filename):
         module=module,
         function_decl_codes=function_decl_codes,
         function_body_codes=function_body_codes,
-        module_const_blob_name=deriveModuleConstantsBlobName(data_filename),
+        module_const_blob_name=encodePythonStringToC(
+            Plugins.deriveModuleConstantsBlobName(data_filename)
+        ),
         context=context,
     )
 
@@ -548,6 +548,7 @@ setExpressionDispatchDict(
         "EXPRESSION_BUILTIN_SUPER2": generateBuiltinSuperCode,
         "EXPRESSION_BUILTIN_ISINSTANCE": generateBuiltinIsinstanceCode,
         "EXPRESSION_BUILTIN_ISSUBCLASS": generateBuiltinIssubclassCode,
+        "EXPRESSION_TYPE_CHECK": generateTypeCheckCode,
         "EXPRESSION_BUILTIN_DIR1": generateBuiltinDir1Code,
         "EXPRESSION_BUILTIN_VARS": generateBuiltinVarsCode,
         "EXPRESSION_BUILTIN_HASATTR": generateBuiltinHasattrCode,
@@ -575,9 +576,9 @@ setExpressionDispatchDict(
         "EXPRESSION_CALL_KEYWORDS_ONLY": generateCallCode,
         "EXPRESSION_CALL_NO_KEYWORDS": generateCallCode,
         "EXPRESSION_CALL": generateCallCode,
-        "EXPRESSION_CONSTANT_NONE_REF": generateConstantNoneReferenceCode,
-        "EXPRESSION_CONSTANT_TRUE_REF": generateConstantTrueReferenceCode,
-        "EXPRESSION_CONSTANT_FALSE_REF": generateConstantFalseReferenceCode,
+        "EXPRESSION_CONSTANT_NONE_REF": generateConstantReferenceCode,
+        "EXPRESSION_CONSTANT_TRUE_REF": generateConstantReferenceCode,
+        "EXPRESSION_CONSTANT_FALSE_REF": generateConstantReferenceCode,
         "EXPRESSION_CONSTANT_STR_REF": generateConstantReferenceCode,
         "EXPRESSION_CONSTANT_STR_EMPTY_REF": generateConstantReferenceCode,
         "EXPRESSION_CONSTANT_UNICODE_REF": generateConstantReferenceCode,
@@ -588,7 +589,7 @@ setExpressionDispatchDict(
         "EXPRESSION_CONSTANT_LONG_REF": generateConstantReferenceCode,
         "EXPRESSION_CONSTANT_FLOAT_REF": generateConstantReferenceCode,
         "EXPRESSION_CONSTANT_COMPLEX_REF": generateConstantReferenceCode,
-        "EXPRESSION_CONSTANT_ELLIPSIS_REF": generateConstantEllipsisReferenceCode,
+        "EXPRESSION_CONSTANT_ELLIPSIS_REF": generateConstantReferenceCode,
         "EXPRESSION_CONSTANT_DICT_REF": generateConstantReferenceCode,
         "EXPRESSION_CONSTANT_DICT_EMPTY_REF": generateConstantReferenceCode,
         "EXPRESSION_CONSTANT_TUPLE_REF": generateConstantReferenceCode,
