@@ -30,21 +30,20 @@ from nuitka.build import SconsInterface
 from nuitka.Options import assumeYesForDownloads, getIconPaths, getJobLimit
 from nuitka.OutputDirectories import getResultBasepath, getResultFullpath
 from nuitka.plugins.Plugins import Plugins
-from nuitka.PostProcessing import version_resources
+from nuitka.PostProcessing import (
+    executePostProcessingResources,
+    version_resources,
+)
 from nuitka.Tracing import general, postprocessing_logger, scons_logger
 from nuitka.utils.Download import getCachedDownload
-from nuitka.utils.Execution import (
-    getNullInput,
-    getNullOutput,
-    withEnvironmentVarsOverriden,
-)
+from nuitka.utils.Execution import getNullInput, withEnvironmentVarsOverriden
 from nuitka.utils.FileOperations import (
     addFileExecutablePermission,
     getFileList,
     removeDirectory,
 )
 from nuitka.utils.SharedLibraries import locateDLL
-from nuitka.utils.Utils import getArchitecture, getOS
+from nuitka.utils.Utils import getArchitecture, getOS, hasOnefileSupportedOS
 
 
 def packDistFolderToOnefile(dist_dir, binary_filename):
@@ -64,7 +63,7 @@ def packDistFolderToOnefile(dist_dir, binary_filename):
     Plugins.onOnefileFinished(onefile_output_filename)
 
 
-def getAppImageToolPath():
+def _getAppImageToolPath(for_operation, assume_yes_for_downloads):
     """Return the path of appimagetool (for Linux).
 
     Will prompt the user to download if not already cached in AppData
@@ -85,8 +84,10 @@ def getAppImageToolPath():
         message="""\
 Nuitka will make use of AppImage (https://appimage.org/) tool
 to combine Nuitka dist folder to onefile binary.""",
-        reject="Nuitka does not work in --onefile on Linux without.",
-        assume_yes_for_downloads=assumeYesForDownloads(),
+        reject="Nuitka does not work in --onefile on Linux without."
+        if for_operation
+        else None,
+        assume_yes_for_downloads=assume_yes_for_downloads,
     )
 
 
@@ -99,7 +100,9 @@ def packDistFolderToOnefileLinux(onefile_output_filename, dist_dir, binary_filen
 
     if not locateDLL("fuse"):
         postprocessing_logger.sysexit(
-            "Error, the fuse library (libfuse.so.x) must be installed for onefile to work on Linux."
+            """\
+Error, the fuse library (libfuse.so.x from fuse2, *not* fuse3) must be installed
+for onefile creation to work on Linux."""
         )
 
     # This might be possible to avoid being done with --runtime-file.
@@ -142,10 +145,18 @@ Categories=Utility;"""
         "Creating single file from dist folder, this may take a while."
     )
 
+    stdout_filename = binary_filename + ".appimage.stdout.txt"
+    stderr_filename = binary_filename + ".appimage.stderr.txt"
+
+    stdout_file = open(stdout_filename, "wb")
+    stderr_file = open(stderr_filename, "wb")
+
     # Starting the process while locked, so file handles are not duplicated.
     appimagetool_process = subprocess.Popen(
         (
-            getAppImageToolPath(),
+            _getAppImageToolPath(
+                for_operation=True, assume_yes_for_downloads=assumeYesForDownloads()
+            ),
             dist_dir,
             "--comp",
             "xz",
@@ -154,19 +165,25 @@ Categories=Utility;"""
         ),
         shell=False,
         stdin=getNullInput(),
-        stderr=getNullOutput(),
-        stdout=getNullOutput(),
+        stdout=stdout_file,
+        stderr=stderr_file,
     )
 
     # TODO: Exit code should be checked.
     result = appimagetool_process.wait()
 
+    stdout_file.close()
+    stderr_file.close()
+
     if not os.path.exists(onefile_output_filename):
         postprocessing_logger.sysexit(
-            "Error, expected output file %s not created by AppImage."
-            % onefile_output_filename
+            "Error, expected output file %r not created by AppImage, check its outputs %r and %r."
+            % (onefile_output_filename, stdout_filename, stderr_filename)
         )
 
+    os.unlink(stdout_filename)
+    os.unlink(stderr_filename)
+
     postprocessing_logger.info("Completed onefile creation.")
 
     assert result == 0, result
@@ -197,8 +214,12 @@ def _runOnefileScons(quiet):
 
     onefile_env_values = {}
 
-    if not Options.isWindowsOnefileTempDirMode():
-        # Merge version information if necessary, to avoid collisions, or deep nesting
+    if Options.isWindowsOnefileTempDirMode():
+        onefile_env_values["ONEFILE_TEMP_SPEC"] = Options.getWindowsOnefileTempDirSpec(
+            use_default=True
+        )
+    else:
+        # Merge version information if possible, to avoid collisions, or deep nesting
         # in file system.
         product_version = version_resources["ProductVersion"]
         file_version = version_resources["FileVersion"]
@@ -254,6 +275,8 @@ def packDistFolderToOnefileWindows(onefile_output_filename, dist_dir):
     # First need to create the bootstrap binary for unpacking.
     _runOnefileScons(quiet=not Options.isShowScons())
 
+    executePostProcessingResources(manifest=None, onefile=True)
+
     # Now need to append to payload it, potentially compressing it.
     compression_indicator, compressor = _pickCompressor()
 
@@ -268,7 +291,7 @@ def packDistFolderToOnefileWindows(onefile_output_filename, dist_dir):
 
         # Move the binary to start immediately to the start position
         start_binary = getResultFullpath(onefile=False)
-        file_list = getFileList(dist_dir)
+        file_list = getFileList(dist_dir, normalize=False)
         file_list.remove(start_binary)
         file_list.insert(0, start_binary)
 
@@ -288,3 +311,14 @@ def packDistFolderToOnefileWindows(onefile_output_filename, dist_dir):
         output_file.write(b"\0\0")
 
         output_file.write(struct.pack("Q", start_pos))
+
+
+def checkOnefileReadiness(assume_yes_for_downloads):
+    if getOS() == "Linux":
+        app_image_path = _getAppImageToolPath(
+            for_operation=False, assume_yes_for_downloads=assume_yes_for_downloads
+        )
+
+        return app_image_path is not None
+    else:
+        return hasOnefileSupportedOS()
