@@ -1,4 +1,4 @@
-#     Copyright 2020, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -26,16 +26,25 @@ import hashlib
 import os
 import re
 import shutil
+import signal
 import subprocess
 import sys
 import tempfile
+import threading
+import time
 from contextlib import contextmanager
 from optparse import OptionGroup, OptionParser
 
 from nuitka.freezer.DependsExe import getDependsExePath
 from nuitka.Tracing import OurLogger, my_print
+from nuitka.tree.SourceReading import readSourceCodeFromFilename
 from nuitka.utils.AppDirs import getCacheDir
-from nuitka.utils.Execution import check_output, withEnvironmentVarOverriden
+from nuitka.utils.Execution import (
+    check_output,
+    getNullInput,
+    isExecutableCommand,
+    withEnvironmentVarOverriden,
+)
 from nuitka.utils.FileOperations import (
     getFileContentByLine,
     getFileContents,
@@ -53,6 +62,8 @@ from .SearchModes import (
     SearchModeResume,
 )
 
+test_logger = OurLogger("", base_style="blue")
+
 
 def check_result(*popenargs, **kwargs):
     if "stdout" in kwargs:
@@ -73,6 +84,7 @@ def goMainDir():
     os.chdir(os.path.dirname(os.path.abspath(sys.modules["__main__"].__file__)))
 
 
+_python_version_str = None
 _python_version = None
 _python_arch = None
 _python_executable = None
@@ -119,31 +131,33 @@ print("Anaconda" if os.path.exists(os.path.join(sys.prefix, 'conda-meta')) else
         stderr=subprocess.STDOUT,
     )
 
-    global _python_version, _python_arch, _python_executable, _python_vendor  # singleton, pylint: disable=global-statement
+    global _python_version_str, _python_version, _python_arch, _python_executable, _python_vendor  # singleton, pylint: disable=global-statement
 
-    _python_version = version_output.split(b"\n")[0].strip()
+    _python_version_str = version_output.split(b"\n")[0].strip()
     _python_arch = version_output.split(b"\n")[1].strip()
     _python_executable = version_output.split(b"\n")[2].strip()
     _python_vendor = version_output.split(b"\n")[3].strip()
 
-    if sys.version.startswith("3"):
+    if str is not bytes:
+        _python_version_str = _python_version_str.decode("utf-8")
         _python_arch = _python_arch.decode("utf-8")
-        _python_version = _python_version.decode("utf-8")
         _python_executable = _python_executable.decode("utf-8")
         _python_vendor = _python_vendor.decode("utf-8")
 
-    if not silent:
-        my_print("Using concrete python", _python_version, "on", _python_arch)
-
-    assert type(_python_version) is str, repr(_python_version)
+    assert type(_python_version_str) is str, repr(_python_version_str)
     assert type(_python_arch) is str, repr(_python_arch)
     assert type(_python_executable) is str, repr(_python_executable)
 
+    if not silent:
+        my_print("Using concrete python", _python_version_str, "on", _python_arch)
+
     if "COVERAGE_FILE" not in os.environ:
         os.environ["COVERAGE_FILE"] = os.path.join(
             os.path.dirname(__file__), "..", "..", "..", ".coverage"
         )
 
+    _python_version = tuple(int(d) for d in _python_version_str.split("."))
+
     return _python_version
 
 
@@ -155,6 +169,10 @@ def getPythonVendor():
     return _python_vendor
 
 
+def getPythonVersionString():
+    return _python_version_str
+
+
 tmp_dir = None
 
 
@@ -256,8 +274,7 @@ def decideFilenameVersionSkip(filename):
     # This will make many decisions with immediate returns.
     # pylint: disable=too-many-branches,too-many-return-statements
 
-    assert type(filename) is str
-    assert type(_python_version) is str
+    assert type(filename) is str, repr(filename)
 
     # Skip runner scripts by default.
     if filename.startswith("run_"):
@@ -267,54 +284,54 @@ def decideFilenameVersionSkip(filename):
         filename = filename[:-3]
 
     # Skip tests that require Python 2.7 at least.
-    if filename.endswith("27.py") and _python_version.startswith("2.6"):
+    if filename.endswith("27.py") and _python_version < (2, 7):
         return False
 
     # Skip tests that require Python 2 at maximum.
-    if filename.endswith("_2.py") and _python_version.startswith("3"):
+    if filename.endswith("_2.py") and _python_version > (3,):
         return False
 
     # Skip tests that require Python 3.7 at maximum.
-    if filename.endswith("_37.py") and _python_version >= "3.8":
+    if filename.endswith("_37.py") and _python_version > (3, 8):
         return False
 
     # Skip tests that require Python 3.2 at least.
-    if filename.endswith("32.py") and _python_version < "3.2":
+    if filename.endswith("32.py") and _python_version < (3, 2):
         return False
 
     # Skip tests that require Python 3.3 at least.
-    if filename.endswith("33.py") and _python_version < "3.3":
+    if filename.endswith("33.py") and _python_version < (3, 3):
         return False
 
     # Skip tests that require Python 3.4 at least.
-    if filename.endswith("34.py") and _python_version < "3.4":
+    if filename.endswith("34.py") and _python_version < (3, 4):
         return False
 
     # Skip tests that require Python 3.5 at least.
-    if filename.endswith("35.py") and _python_version < "3.5":
+    if filename.endswith("35.py") and _python_version < (3, 5):
         return False
 
     # Skip tests that require Python 3.6 at least.
-    if filename.endswith("36.py") and _python_version < "3.6":
+    if filename.endswith("36.py") and _python_version < (3, 6):
         return False
 
     # Skip tests that require Python 3.7 at least.
-    if filename.endswith("37.py") and _python_version < "3.7":
+    if filename.endswith("37.py") and _python_version < (3, 7):
         return False
 
     # Skip tests that require Python 3.8 at least.
-    if filename.endswith("38.py") and _python_version < "3.8":
+    if filename.endswith("38.py") and _python_version < (3, 8):
         return False
 
     # Skip tests that require Python 3.9 at least.
-    if filename.endswith("39.py") and _python_version < "3.9":
+    if filename.endswith("39.py") and _python_version < (3, 9):
         return False
 
     return True
 
 
 def decideNeeds2to3(filename):
-    return _python_version.startswith("3") and not filename.endswith(
+    return _python_version >= (3,) and not filename.endswith(
         ("32.py", "33.py", "34.py", "35.py", "36.py", "37.py", "38.py", "39.py")
     )
 
@@ -456,28 +473,36 @@ def hasDebugPython():
     return False
 
 
-def isExecutableCommand(command):
-    path = os.environ["PATH"]
+def displayRuntimeTraces(logger, path):
+    if not os.path.exists(path):
+        # TODO: Have a logger package passed.
+        logger.sysexit("Error, cannot find %r (%r)." % (path, os.path.abspath(path)))
 
-    suffixes = (".exe",) if os.name == "nt" else ("",)
+    path = os.path.abspath(path)
 
-    for part in path.split(os.pathsep):
-        if not part:
-            continue
+    # TODO: Merge code for building command with below function, this is otherwise
+    # horribly bad.
 
-        for suffix in suffixes:
-            if os.path.isfile(os.path.join(part, command + suffix)):
-                return True
-
-    return False
+    if os.name == "posix":
+        # Run with traces to help debugging, specifically in CI environment.
+        if sys.platform == "darwin" or sys.platform.startswith("freebsd"):
+            test_logger.info("dtruss:")
+            os.system("sudo dtruss %s" % path)
+        else:
+            test_logger.info("strace:")
+            os.system("strace -s4096 -e file %s" % path)
 
 
-def getRuntimeTraceOfLoadedFiles(path, trace_stderr=True):
+def getRuntimeTraceOfLoadedFiles(logger, path):
     """ Returns the files loaded when executing a binary. """
 
     # This will make a crazy amount of work,
     # pylint: disable=I0021,too-many-branches,too-many-locals,too-many-statements
 
+    if not os.path.exists(path):
+        # TODO: Have a logger package passed.
+        logger.sysexit("Error, cannot find %r (%r)." % (path, os.path.abspath(path)))
+
     result = []
 
     if os.name == "posix":
@@ -506,13 +531,15 @@ Error, needs 'strace' on your system to scan used libraries."""
                 "strace",
                 "-e",
                 "file",
-                "-s4096",  # Some paths are truncated otherwise.
-                path,
+                "-s4096",  # Some paths are truncated in output otherwise
+                os.path.abspath(path),
             )
 
         # Ensure executable is not polluted with third party stuff,
         # tests may fail otherwise due to unexpected libs being loaded
         with withEnvironmentVarOverriden("LD_PRELOAD", None):
+            tracing_command = args[0] if args[0] != "sudo" else args[1]
+
             process = subprocess.Popen(
                 args=args, stdout=subprocess.PIPE, stderr=subprocess.PIPE
             )
@@ -524,15 +551,15 @@ Error, needs 'strace' on your system to scan used libraries."""
                 if str is not bytes:
                     stderr_strace = stderr_strace.decode("utf8")
 
-                my_print(stderr_strace, file=sys.stderr)
-                sys.exit("Failed to run strace.")
+                logger.warning(stderr_strace)
+                logger.sysexit("Failed to run %r." % tracing_command)
 
             with open(path + ".strace", "wb") as f:
                 f.write(stderr_strace)
 
             for line in stderr_strace.split(b"\n"):
-                if process.returncode != 0 and trace_stderr:
-                    my_print(line)
+                if process.returncode != 0:
+                    logger.my_print(line)
 
                 if not line:
                     continue
@@ -560,7 +587,8 @@ Error, needs 'strace' on your system to scan used libraries."""
                         continue
 
                     if filename in (
-                        b"/usr/bin/python3." + version for version in (b"5", b"6", b"7")
+                        b"/usr/bin/python3." + version
+                        for version in (b"5", b"6", b"7", b"8", b"9")
                     ):
                         continue
 
@@ -580,7 +608,11 @@ Error, needs 'strace' on your system to scan used libraries."""
                         binary_path = os.path.dirname(binary_path)
 
                         if filename == os.path.join(
-                            binary_path, b"python" + _python_version[:3].encode("utf8")
+                            binary_path,
+                            b"python"
+                            + (
+                                "%d%d" % (_python_version[0], _python_version[1])
+                            ).encode("utf8"),
                         ):
                             found = True
                             continue
@@ -596,20 +628,28 @@ Error, needs 'strace' on your system to scan used libraries."""
             if sys.version.startswith("3"):
                 result = [s.decode("utf-8") for s in result]
     elif os.name == "nt":
-        subprocess.call(
-            (
-                getDependsExePath(),
-                "-c",
-                "-ot%s" % path + ".depends",
-                "-f1",
-                "-pa1",
-                "-ps1",
-                "-pp0",
-                "-pl1",
-                path,
-            )
+        command = (
+            getDependsExePath(),
+            "-c",  # Console mode
+            "-ot%s" % path + ".depends",
+            "-f1",
+            "-pb",
+            "-pa1",  # Turn on all profiling options.
+            "-ps1",  # Simulate ShellExecute with app dirs in PATH.
+            "-pp1",  # Do not long DllMain calls.
+            "-po1",  # Log DllMain call for all other messages.
+            "-ph1",  # Hook the process.
+            "-pl1",  # Log LoadLibrary calls.
+            "-pt1",  # Thread information.
+            "-pe1",  # First chance exceptions.
+            "-pg1",  # Log GetProcAddress calls.
+            "-pf1",  # Use full paths.
+            "-pc1",  # Profile child processes.
+            path,
         )
 
+        subprocess.call(command)
+
         inside = False
         for line in getFileContentByLine(path + ".depends"):
             if "| Module Dependency Tree |" in line:
@@ -676,7 +716,7 @@ def checkRuntimeLoadedFilesForOutsideAccesses(loaded_filenames, white_list):
         if ok:
             continue
 
-        if loaded_filename.startswith(("/etc/", "/usr/etc")):
+        if loaded_filename.startswith(("/etc/", "/usr/etc", "/usr/local/etc")):
             continue
 
         if loaded_filename.startswith("/proc/") or loaded_filename == "/proc":
@@ -771,7 +811,7 @@ def checkRuntimeLoadedFilesForOutsideAccesses(loaded_filenames, white_list):
 def hasModule(module_name):
     with open(os.devnull, "w") as devnull:
         result = subprocess.call(
-            (os.environ["PYTHON"], "-c" "import %s" % module_name),
+            (os.environ["PYTHON"], "-c", "import %s" % module_name),
             stdout=devnull,
             stderr=subprocess.STDOUT,
         )
@@ -968,6 +1008,8 @@ def checkReferenceCount(checked_function, max_rounds=20, explain=False):
 
 
 def createSearchMode():
+    # Dealing with many options, pylint: disable=too-many-branches
+
     parser = OptionParser()
 
     select_group = OptionGroup(parser, "Select Tests")
@@ -1003,10 +1045,25 @@ Executing all self checks possible to find errors in Nuitka, good for test cover
 Defaults to off.""",
     )
 
+    debug_group.add_option(
+        "--commands",
+        action="store_true",
+        dest="show_commands",
+        default=False,
+        help="""Output commands being done in output comparison.
+Defaults to off.""",
+    )
+
     parser.add_option_group(debug_group)
 
     options, positional_args = parser.parse_args()
 
+    if options.debug:
+        addExtendedExtraOptions("--debug")
+
+    if options.show_commands:
+        os.environ["NUITKA_TRACE_COMMANDS"] = "1"
+
     # Default to searching.
     mode = positional_args[0] if positional_args else "search"
 
@@ -1138,10 +1195,8 @@ def withPythonPathChange(python_path):
             os.environ["PYTHONPATH"] = old_path
 
 
-@contextmanager
-def withExtendedExtraOptions(*args):
-    assert args
-    old_value = os.environ.get("NUITKA_EXTRA_OPTIONS", None)
+def addExtendedExtraOptions(*args):
+    old_value = os.environ.get("NUITKA_EXTRA_OPTIONS")
 
     value = old_value
 
@@ -1153,6 +1208,15 @@ def withExtendedExtraOptions(*args):
 
     os.environ["NUITKA_EXTRA_OPTIONS"] = value
 
+    return old_value
+
+
+@contextmanager
+def withExtendedExtraOptions(*args):
+    assert args
+
+    old_value = addExtendedExtraOptions(*args)
+
     yield
 
     if old_value is None:
@@ -1404,18 +1468,6 @@ def getTestingCPythonOutputsCacheDir():
     return result
 
 
-@contextmanager
-def withDirectoryChange(path, allow_none=False):
-    if path is not None or not allow_none:
-        old_cwd = os.getcwd()
-        os.chdir(path)
-
-    yield
-
-    if path is not None or not allow_none:
-        os.chdir(old_cwd)
-
-
 def scanDirectoryForTestCases(dirname, template_context=None):
     filenames = os.listdir(dirname)
 
@@ -1469,7 +1521,10 @@ def setupCacheHashSalt(test_code_path):
         git_cmd = ["git", "ls-tree", "-r", "HEAD", test_code_path]
 
         process = subprocess.Popen(
-            args=git_cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE
+            args=git_cmd,
+            stdin=getNullInput(),
+            stdout=subprocess.PIPE,
+            stderr=subprocess.PIPE,
         )
 
         stdout_git, stderr_git = process.communicate()
@@ -1486,6 +1541,30 @@ def setupCacheHashSalt(test_code_path):
     os.environ["NUITKA_HASH_SALT"] = salt_value.hexdigest()
 
 
+def displayFolderContents(name, path):
+    test_logger.info("Listing of %s %r:" % (name, path))
+
+    if os.path.exists(path):
+        if os.name == "nt":
+            command = "dir /b /s /a:-D %s" % path
+        else:
+            command = "ls -Rla %s" % path
+
+        os.system(command)
+    else:
+        test_logger.info("Does not exist.")
+
+
+def displayFileContents(name, path):
+    test_logger.info("Contents of %s %r:" % (name, path))
+
+    if os.path.exists(path):
+        for line in getFileContentByLine(path):
+            my_print(line)
+    else:
+        test_logger.info("Does not exist.")
+
+
 def someGenerator():
     yield 1
     yield 2
@@ -1497,4 +1576,69 @@ def someGeneratorRaising():
     raise TypeError(2)
 
 
-test_logger = OurLogger("", base_style="blue")
+# checks requirements needed to run each test module, according to the specified special comment
+# special comments are in the following formats:
+#     "# nuitka-skip-unless-expression: expression to be evaluated"
+#       OR
+#     "# nuitka-skip-unless-imports: module1,module2,..."
+def checkRequirements(filename):
+    for line in readSourceCodeFromFilename(None, filename).splitlines():
+        if line.startswith("# nuitka-skip-unless-"):
+            if line[21:33] == "expression: ":
+                expression = line[33:]
+                with open(os.devnull, "w") as devnull:
+                    result = subprocess.call(
+                        (
+                            os.environ["PYTHON"],
+                            "-c",
+                            "import sys, os; sys.exit(not bool(%s))" % expression,
+                        ),
+                        stdout=devnull,
+                        stderr=subprocess.STDOUT,
+                    )
+                if result != 0:
+                    return (False, "Expression '%s' evaluated to false" % expression)
+
+            elif line[21:30] == "imports: ":
+                imports_needed = line[30:].rstrip().split(",")
+                for i in imports_needed:
+                    if not hasModule(i):
+                        return (
+                            False,
+                            i
+                            + " not installed for this Python version, but test needs it",
+                        )
+    # default return value
+    return (True, "")
+
+
+class DelayedExecutionThread(threading.Thread):
+    def __init__(self, timeout, func):
+        threading.Thread.__init__(self)
+        self.timeout = timeout
+
+        self.func = func
+
+    def run(self):
+        time.sleep(self.timeout)
+        self.func()
+
+
+def executeAfterTimePassed(timeout, func):
+    alarm = DelayedExecutionThread(timeout=timeout, func=func)
+    alarm.start()
+
+
+def killProcess(name, pid):
+    """Kill a process in a portable way.
+
+    Right now SIGINT is used, unclear what to do on Windows
+    with Python2 or non-related processes.
+    """
+
+    if str is bytes and os.name == "nt":
+        test_logger.info("Using taskkill on test process %r." % name)
+        os.system("taskkill.exe /PID %d" % pid)
+    else:
+        test_logger.info("Killing test process %r." % name)
+        os.kill(pid, signal.SIGINT)
